package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinition;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinitionLimit;
import com.jpsoft.picc.modules.base.entity.InsuranceJobs;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionLimitService;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionService;
import com.jpsoft.picc.modules.base.service.InsuranceJobsService;
import com.jpsoft.picc.modules.business.entity.StudentApplication;
import com.jpsoft.picc.modules.business.entity.StudentApplicationRecord;
import com.jpsoft.picc.modules.business.service.StudentApplicationRecordService;
import com.jpsoft.picc.modules.business.service.StudentApplicationService;
import com.jpsoft.picc.modules.common.constant.StudentPolicyStatus;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/studentInsurance")
@Api(description = "险种信息")
public class StudentInsuranceController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsuranceDefinitionService insuranceDefinitionService;
    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;
    @Autowired
    private InsuranceJobsService insuranceJobsService;
    @Autowired
    private StudentApplicationService studentApplicationService;
    @Autowired
    private StudentApplicationRecordService studentApplicationRecordService;

    @ApiOperation(value="获取保险信息")
    @PostMapping("detail")

    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "险种编号", required = true, paramType = "form")
    })
    public MessageResult<Map> detail(String id){
        MessageResult<Map> msgResult = new MessageResult<>();
        Map<String,Object> map = new HashMap<>();

        try {
            InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(id);

            map.put("致家长的一封信",insuranceDefinition.getLetter());
            map.put("保障计划",getLimitList(insuranceDefinition.getId(),false,"3a1da4e2-7274-414b-bbc5-c0b37032db17"));
            map.put("特别声明",insuranceDefinition.getSpecialStatement());
            map.put("保险条款",insuranceDefinition.getClause());
            map.put("投保声明",insuranceDefinition.getInsuranceDescription());
            map.put("责任免除说明",insuranceDefinition.getExemptionInstructions());
            map.put("保险责任文本",insuranceDefinition.getResponsibility());
            if(insuranceDefinition.getHaveAttach()) {
                map.put("附加险保险责任文本", insuranceDefinition.getResponsibilityAttach());
                map.put("附加险保障计划",getLimitList(insuranceDefinition.getId(),true,"98f3ac96-bdf6-45d5-acca-c4561baeaf20"));
            }

            msgResult.setResult(true);
            msgResult.setData(map);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    private List<Map<String,Object>> getLimitList(String definitionId, Boolean isAttach, String JobsId){
        List<Map<String,Object>> list = new ArrayList<>();

        List<InsuranceDefinitionLimit> insuranceDefinitionLimitList = insuranceDefinitionLimitService.findByDefinitionIdAndIsAttach(definitionId,isAttach);
        for(InsuranceDefinitionLimit insuranceDefinitionLimit : insuranceDefinitionLimitList){
            Map<String,Object> map = new HashMap<>();
            map.put("name",insuranceDefinitionLimit.getName());
            map.put("limit",insuranceDefinitionLimit.getLimit());

            list.add(map);
        }

        List<InsuranceJobs> insuranceJobs = insuranceJobsService.findByDefinitionIdAndJobId(definitionId,JobsId);
        Map<String,Object> map1 = new HashMap<>();
        map1.put("name","承保年龄");
        map1.put("limit","3-18周岁");
        list.add(map1);
        Map<String,Object> map2 = new HashMap<>();
        map2.put("name","保险期限");
        map2.put("limit","1年");
        list.add(map2);
        Map<String,Object> map3 = new HashMap<>();
        map3.put("name","保费金额");
        map3.put("limit",insuranceJobs.get(0).getCharges());
        list.add(map3);

        return list;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("save")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = false, paramType = "query"),
            @ApiImplicitParam(name = "insuredId",value = "学生编号", required = false, paramType = "query"),
            @ApiImplicitParam(name = "insuredName",value = "学生姓名", required = false, paramType = "query"),
            @ApiImplicitParam(name = "insuredSex",value = "学生性别", required = false,paramType="query"),
            @ApiImplicitParam(name = "insuredSchoolId",value = "学校编号", required = false, paramType = "query"),
            @ApiImplicitParam(name = "insuredSchoolName",value = "学校", required = false, paramType = "query"),
            @ApiImplicitParam(name = "companyId",value = "班级编号", required = false,paramType="query"),
            @ApiImplicitParam(name = "companyName",value = "班级", required = false, paramType = "query"),
            @ApiImplicitParam(name = "insuredCard",value = "学生身份证号", required = false, paramType = "query"),
            @ApiImplicitParam(name = "haveMedicare",value = "有无医保", required = false,paramType="query"),
            @ApiImplicitParam(name = "attachId",value = "家长编号", required = false, paramType = "query"),
            @ApiImplicitParam(name = "attachName",value = "家长姓名", required = false, paramType = "query"),
            @ApiImplicitParam(name = "relationship",value = "亲属关系", required = false,paramType="query"),
            @ApiImplicitParam(name = "attachTel",value = "家长电话", required = false, paramType = "query"),
            @ApiImplicitParam(name = "definitionId",value = "保障方案编号", required = false, paramType = "query"),
            @ApiImplicitParam(name = "definitionName",value = "保障方案", required = false,paramType="query"),
            @ApiImplicitParam(name = "startTime",value = "生效时间", required = false, paramType = "query"),
            @ApiImplicitParam(name = "endTime",value = "失效时间", required = false, paramType = "query")
    })
    public MessageResult<String> save(
            String id,String insuredId,String insuredName,String insuredSex,String insuredSchoolId,String insuredSchoolName,String companyId,String companyName,String insuredCard,Boolean haveMedicare,
            String attachId,String attachName,String relationship,String attachTel,
            String definitionId,String definitionName,
            String startTime,String endTime,
            @RequestAttribute String subject){
        MessageResult<String> msgResult = new MessageResult<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmssSSS");
        StudentApplication studentApplication = null;
        InsuranceDefinition insuranceDefinition = insuranceDefinitionService.findByName(definitionName);

        try {
            if(StringUtils.isEmpty(id)) {
                studentApplication = new StudentApplication();
                studentApplication.setId(UUID.randomUUID().toString());
                studentApplication.setInsuredId(insuredId);
                studentApplication.setInsuredName(insuredName);
                studentApplication.setInsuredSex(insuredSex);
                studentApplication.setSchoolId(insuredSchoolId);
                studentApplication.setSchoolName(insuredSchoolName);
                studentApplication.setCompanyId(companyId);
                studentApplication.setCompanyName(companyName);
                studentApplication.setInsuredCard(insuredCard);
                studentApplication.setHaveMedicare(haveMedicare);
                studentApplication.setAttachId(attachId);
                studentApplication.setAttachName(attachName);
                studentApplication.setAttachRelationship(relationship);
                studentApplication.setAttachTel(attachTel);
                studentApplication.setDefinitionId(definitionId);
                studentApplication.setDefinitionName(definitionName);
                if (insuranceDefinition.getHaveAttach()) {
                    studentApplication.setHaveAttach(true);

                    String num = sdf.format(new Date());
                    studentApplication.setPolicyNo(num + "," + num + "F");
                } else {
                    studentApplication.setHaveAttach(false);

                    String num = sdf.format(new Date());
                    studentApplication.setPolicyNo(num);
                }
                studentApplication.setStartTime(sdf.parse(startTime));
                studentApplication.setEndTime(sdf.parse(endTime));
                studentApplication.setDelFlag(false);
                studentApplication.setCreateBy(subject);
                studentApplication.setCreateTime(new Date());
                studentApplication.setStatus(String.valueOf(StudentPolicyStatus.PendingPay.getValue()));
                studentApplication.setProcessStatus("正常");

                int affectCount = studentApplicationService.insert(studentApplication);

                if (affectCount > 0) {
                    StudentApplicationRecord studentApplicationRecord = new StudentApplicationRecord();
                    studentApplicationRecord.setId(UUID.randomUUID().toString());
                    studentApplicationRecord.setApplicationId(studentApplication.getId());
                    studentApplicationRecord.setProcessStatus("正常");
                    studentApplicationRecord.setStatus(String.valueOf(StudentPolicyStatus.PendingPay.getValue()));
                    studentApplicationRecord.setDelFlag(false);
                    studentApplicationRecord.setCreateBy(subject);
                    studentApplicationRecord.setCreateTime(new Date());
                    studentApplicationRecordService.insert(studentApplicationRecord);

                    msgResult.setResult(true);
                    msgResult.setData(studentApplication.getId());
                } else {
                    msgResult.setResult(false);
                    msgResult.setMessage("数据库添加失败");
                }
            }else {
                studentApplication = studentApplicationService.get(id);
                studentApplication.setInsuredId(insuredId);
                studentApplication.setInsuredName(insuredName);
                studentApplication.setInsuredSex(insuredSex);
                studentApplication.setSchoolId(insuredSchoolId);
                studentApplication.setSchoolName(insuredSchoolName);
                studentApplication.setCompanyId(companyId);
                studentApplication.setCompanyName(companyName);
                studentApplication.setInsuredCard(insuredCard);
                studentApplication.setHaveMedicare(haveMedicare);
                studentApplication.setAttachId(attachId);
                studentApplication.setAttachName(attachName);
                studentApplication.setAttachRelationship(relationship);
                studentApplication.setAttachTel(attachTel);
                studentApplication.setDefinitionId(definitionId);
                studentApplication.setDefinitionName(definitionName);
                if (insuranceDefinition.getHaveAttach()) {
                    studentApplication.setHaveAttach(true);

                    String num = sdf.format(new Date());
                    studentApplication.setPolicyNo(num + "," + num + "F");
                } else {
                    studentApplication.setHaveAttach(false);

                    String num = sdf.format(new Date());
                    studentApplication.setPolicyNo(num);
                }
                studentApplication.setStartTime(sdf.parse(startTime));
                studentApplication.setEndTime(sdf.parse(endTime));
                studentApplication.setDelFlag(false);
                studentApplication.setUpdateBy(subject);
                studentApplication.setUpdateTime(new Date());
                studentApplication.setStatus(String.valueOf(StudentPolicyStatus.PendingPay.getValue()));
                if(studentApplication.getProcessStatus().equals("退回")){
                    studentApplication.setProcessStatus("正常");

                    StudentApplicationRecord studentApplicationRecord = new StudentApplicationRecord();
                    studentApplicationRecord.setId(UUID.randomUUID().toString());
                    studentApplicationRecord.setApplicationId(studentApplication.getId());
                    studentApplicationRecord.setProcessStatus("正常");
                    studentApplicationRecord.setStatus(String.valueOf(StudentPolicyStatus.PendingPay.getValue()));
                    studentApplicationRecord.setDelFlag(false);
                    studentApplicationRecord.setCreateBy(subject);
                    studentApplicationRecord.setCreateTime(new Date());
                    studentApplicationRecordService.insert(studentApplicationRecord);
                }
                studentApplicationService.update(studentApplication);

                msgResult.setCode(200);
                msgResult.setResult(true);
                msgResult.setData(studentApplication.getId());
                msgResult.setMessage("成功");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setCode(500);
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="保单列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="insuredCard",value = "投保人身份证",paramType = "query")
    })
    public MessageResult<Map> list(
            String insuredCard,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();
        searchParams.put("delFlag",false);
        if (StringUtils.isNotEmpty(insuredCard)) {
            searchParams.put("insuredCard",insuredCard);
        }

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        Page<StudentApplication> page = studentApplicationService.pageSearch(searchParams,1,1000,false,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="获取订单信息")
    @PostMapping("detailStudentApplication")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form")
    })
    public MessageResult<StudentApplication> detailStudentApplication(String id){
        MessageResult<StudentApplication> msgResult = new MessageResult<>();

        try {
            StudentApplication studentApplication = studentApplicationService.get(id);

            msgResult.setResult(true);
            msgResult.setData(studentApplication);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取订单信息")
    @PostMapping("delete")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form")
    })
    public MessageResult<StudentApplication> delete(String id,@RequestAttribute String subject){
        MessageResult<StudentApplication> msgResult = new MessageResult<>();

        try {
            StudentApplication studentApplication = studentApplicationService.get(id);
            studentApplication.setDelFlag(true);
            studentApplication.setUpdateBy(subject);
            studentApplication.setUpdateTime(new Date());
            studentApplicationService.update(studentApplication);

            msgResult.setResult(true);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="电子签名")
    @PostMapping("signature")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form"),
            @ApiImplicitParam(name = "url",value = "图片路径", required = true, paramType = "form"),
            @ApiImplicitParam(name = "number",value = "第几次签名", required = true, paramType = "form")
    })
    public MessageResult<StudentApplication> signature(String id,String url,Integer number,
                                                       @RequestAttribute String subject){
        MessageResult<StudentApplication> msgResult = new MessageResult<>();

        try {
            StudentApplication studentApplication = studentApplicationService.get(id);

            switch (number){
                case 1:
                    studentApplication.setSignature(url);
                    break;
                case 2:
                    studentApplication.setSignature2(url);
                    break;
                case 3:
                    studentApplication.setSignature3(url);
                    break;
                default:
                    msgResult.setResult(false);
                    msgResult.setMessage("超出签名次数。");
                    return msgResult;
            }

            studentApplication.setDelFlag(true);
            studentApplication.setUpdateBy(subject);
            studentApplication.setUpdateTime(new Date());
            studentApplicationService.update(studentApplication);

            msgResult.setResult(true);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
