package com.jpsoft.picc.modules.business.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinition;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinitionLimit;
import com.jpsoft.picc.modules.base.entity.InsuranceJobs;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionLimitService;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionService;
import com.jpsoft.picc.modules.base.service.InsuranceJobsService;
import com.jpsoft.picc.modules.base.service.TemplateMessageService;
import com.jpsoft.picc.modules.business.entity.*;
import com.jpsoft.picc.modules.business.service.StudentApplicationRecordService;
import com.jpsoft.picc.modules.business.service.StudentApplicationService;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.constant.StudentPolicyStatus;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.utils.NumberToCN;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.utils.VelocityHelper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/business/studentApplication")
@Api(description = "studentApplication")
public class StudentApplicationController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private StudentApplicationService studentApplicationService;

    @Autowired
    private InsuranceDefinitionService insuranceDefinitionService;

    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;

    @Autowired
    private InsuranceJobsService insuranceJobsService;

    @Autowired
    private TemplateMessageService templateMessageService;

    @Autowired
    private StudentApplicationRecordService studentApplicationRecordService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<StudentApplication> create(){
        MessageResult<StudentApplication> msgResult = new MessageResult<>();

        StudentApplication studentApplication = new StudentApplication();

        msgResult.setData(studentApplication);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<StudentApplication> add(@RequestBody StudentApplication studentApplication,@RequestAttribute String subject){
        MessageResult<StudentApplication> msgResult = new MessageResult<>();

        try {
            studentApplication.setId(UUID.randomUUID().toString());
            studentApplication.setDelFlag(false);
            studentApplication.setCreateBy(subject);
            studentApplication.setCreateTime(new Date());
            
            int affectCount = studentApplicationService.insert(studentApplication);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(studentApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<StudentApplication> edit(@PathVariable("id") String id){
        MessageResult<StudentApplication> msgResult = new MessageResult<>();

        try {
            StudentApplication studentApplication = studentApplicationService.get(id);

            if (studentApplication != null) {
                msgResult.setResult(true);
                msgResult.setData(studentApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<StudentApplication> update(@RequestBody StudentApplication studentApplication,@RequestAttribute String subject){
        MessageResult<StudentApplication> msgResult = new MessageResult<>();

        try {
            studentApplication.setUpdateBy(subject);
            studentApplication.setUpdateTime(new Date());
            
            int affectCount = studentApplicationService.update(studentApplication);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(studentApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            StudentApplication studentApplication = studentApplicationService.get(id);
            studentApplication.setDelFlag(true);
            studentApplication.setUpdateBy(subject);
            studentApplication.setUpdateTime(new Date());

            int affectCount = studentApplicationService.update(studentApplication);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                StudentApplication studentApplication = studentApplicationService.get(id);
                studentApplication.setDelFlag(true);
                studentApplication.setUpdateBy(subject);
                studentApplication.setUpdateTime(new Date());

                affectCount += studentApplicationService.update(studentApplication);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            String tabStatus,
            String userType,
            String policyNo,
            String status,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(tabStatus)) {
            searchParams.put("tabStatus",tabStatus);
        }

        if (StringUtils.isNotEmpty(policyNo)) {
            searchParams.put("policyNo","%" + policyNo + "%");
        }

        if (StringUtils.isNotEmpty(userType)) {
            if(userType.equals("1")){
                searchParams.put("userType",userType);
            }
        }

        if (StringUtils.isNotEmpty(status)) {
            searchParams.put("status",status);
        }

        searchParams.put("delFlag",false);
        Page<StudentApplication> page = studentApplicationService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="标签页数量提示")
    @RequestMapping(value = "tabCount",method = RequestMethod.POST)
    public MessageResult<Map> tabCount(
            String tabStatus,
            String userType,
            String policyNo,
            String status,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(tabStatus)) {
            searchParams.put("tabStatus",tabStatus);
        }

        if (StringUtils.isNotEmpty(policyNo)) {
            searchParams.put("policyNo","%" + policyNo + "%");
        }

        if (StringUtils.isNotEmpty(userType)) {
            if(userType.equals("1")){
                searchParams.put("userType",userType);
            }
        }
        if (StringUtils.isNotEmpty(status)) {
            searchParams.put("status",status);
        }

        searchParams.put("delFlag",false);

        Map<String,Object> countMap = new HashMap<>();

        Page<StudentApplication> allPage = studentApplicationService.pageSearch(searchParams,1,100000,true,sortList);

        countMap.put("all",allPage.getTotal());

        for (StudentPolicyStatus sPolicyStatus : StudentPolicyStatus.values()) {
            searchParams.put("status",sPolicyStatus.getValue());
            Page<StudentApplication> page = studentApplicationService.pageSearch(searchParams,1,100000,true,sortList);
            System.out.println(sPolicyStatus.getText() + "=============" + page.getTotal());
            countMap.put(sPolicyStatus.getValue() + "" , page.getTotal());
        }

        msgResult.setResult(true);
        msgResult.setData(countMap);

        return msgResult;
    }


    @ApiOperation(value="添加测试信息")
    @PostMapping("addTestApplication")
    public MessageResult<StudentApplication> addTestApplication(
            String type,
            @RequestAttribute String subject){
        MessageResult<StudentApplication> msgResult = new MessageResult<>();

        try {
            StudentApplication studentApplication = new StudentApplication();
            studentApplication.setId(UUID.randomUUID().toString());
            studentApplication.setDelFlag(false);
            studentApplication.setCreateBy(subject);
            studentApplication.setCreateTime(new Date());

            //学平险 fa84df46-1d38-4aa7-ae15-4c8b28c467ed
            //学平险+监护人责任险 b318c3ac-c460-4d93-aa92-78b57efba125
            if("1".equals(type)){
                studentApplication.setDefinitionId("fa84df46-1d38-4aa7-ae15-4c8b28c467ed");
            }else {
                studentApplication.setDefinitionId("b318c3ac-c460-4d93-aa92-78b57efba125");
            }
            DateTime createTime = new DateTime(new Date());
            //投保单号
            studentApplication.setPolicyNo(createTime.toString("yyyyMMddHHmmssSSS"));
            //投保人ID insured_id
            //投保人姓名 insured_name
            if("1".equals(type)){
                studentApplication.setInsuredName("舒展");
            }else {
                studentApplication.setInsuredName("舒展监护人责任险");
            }
            //投保人证件号码 insured_card
            studentApplication.setInsuredCard("421002199101141033");
            //投保人联系电话 insured_tel
            studentApplication.setInsuredTel("13437120102");
            //是否有附加险 have_attach
            studentApplication.setInsuredSex("1");

            if("1".equals(type)){
                studentApplication.setHaveAttach(false);
            }else {
                studentApplication.setHaveAttach(true);
                //附加险投保人ID attach_id
                //附加险投保人姓名 attach_name
                studentApplication.setAttachRelationship("父亲");
                studentApplication.setAttachName("舒爸爸");
                //附加险投保人证件号码 attach_card
                studentApplication.setAttachCard("421002198101141033");
                //附加险投保人联系电话 attach_tel
                studentApplication.setAttachTel("13476980773");
            }
            //有无医保 have_medicare
            studentApplication.setHaveMedicare(true);
            DateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            //保障开始时间 start_time
            studentApplication.setStartTime(format.parse("2020-08-01"));
            //保障结束时间 end_time
            studentApplication.setEndTime(format.parse("2020-09-01"));
            //保险费合计(总金额) insurance_fee
            if("1".equals(type)){
                studentApplication.setInsuranceFee(new BigDecimal(100));
            }else {
                studentApplication.setInsuranceFee(new BigDecimal(120));
            }
            //状态:待缴费20/待审核30/待制单40/待签名50/已出单60/已过期100 status_
            studentApplication.setStatus("30");
            //流程审核状态:正常/回退等 process_status
            studentApplication.setProcessStatus("正常");
            //出单时间 finish_time
            //studentApplication.setFinishTime(new Date());
            //电子签名 signature_
            studentApplication.setSignature("");
            //电子签名2 signature_2
            studentApplication.setSignature2("");
            //电子签名3 signature_3
            studentApplication.setSignature3("");
            //支付订单号 payment_order
            studentApplication.setPaymentOrder("000000009");

            //学校ID
            studentApplication.setSchoolId("sssyzx");
            //学校名称
            studentApplication.setSchoolName("沙市实验中学");
            //班级ID
            studentApplication.setCompanyId("1n1b");
            //班级名称
            studentApplication.setCompanyName("1年1班");
            int affectCount = studentApplicationService.insert(studentApplication);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(studentApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取投保信息")
    @PostMapping("detail")
    @ApiImplicitParams({
            @ApiImplicitParam(name="id",value = "投保信息ID",required = true,paramType = "query")
    })
    public MessageResult<StudentApplication> detail(String id){
        MessageResult<StudentApplication> msgResult = new MessageResult<>();

        try {
            StudentApplication studentApplication = studentApplicationService.get(id);

            if (studentApplication != null) {
                msgResult.setResult(true);
                msgResult.setData(studentApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取承保信息")
    @RequestMapping(value = "acceptDetail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query")
    })
    public MessageResult<Map> acceptDetail(String applicationId){

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> map = new HashMap<String,Object>();

        //投保单
        StudentApplication studentApplication = studentApplicationService.get(applicationId);

        //险种
        InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(studentApplication.getDefinitionId());

        String definitionId = insuranceDefinition.getId();

        map.put("id",insuranceDefinition.getId());
        map.put("name",insuranceDefinition.getName());

        List<InsuranceDefinitionLimit> insuranceDefinitionLimitList = insuranceDefinitionLimitService.findByDefinitionId(definitionId);

        List<Map<String,Object>> mapList = new ArrayList<Map<String,Object>>();


        Map<String,Object> varsMap = new HashMap<>();
        varsMap.put("num",1);

        for(InsuranceDefinitionLimit  insuranceDefinitionLimit:insuranceDefinitionLimitList){
            Map<String,Object> map1 = new HashMap<String,Object>();
            map1.put("id",insuranceDefinitionLimit.getId());
            map1.put("name",insuranceDefinitionLimit.getName());

            //限额这里可能是公式，需要带入人数
            String limit = VelocityHelper.format(insuranceDefinitionLimit.getLimit(),varsMap);
            map1.put("limit",limit);

            String unit = VelocityHelper.format(insuranceDefinitionLimit.getUnit(),varsMap);
            map1.put("unit",unit);

            mapList.add(map1);
        }

        map.put("insuranceDefinitionLimitList",mapList);

        List<InsuranceJobs> InsuranceJobsList = insuranceJobsService.findByDefinitionId(studentApplication.getDefinitionId());

        List<Map<String,Object>> mapList1 = new ArrayList<Map<String,Object>>();

        DateTime startTime = new DateTime(studentApplication.getStartTime());
        map.put("startTime",startTime.toString("yyyy-MM-dd"));

        DateTime endTime = new DateTime(studentApplication.getEndTime());
        map.put("endTime",endTime.toString("yyyy-MM-dd"));

        //月份
        int months = 1;

        BigDecimal totalAmount = new BigDecimal(0);

        for (InsuranceJobs insuranceJobs:InsuranceJobsList) {
            Map<String,Object> map1 = new HashMap<String,Object>();
            String jobId = insuranceJobs.getJobsId();
            map1.put("id",jobId);
            map1.put("name",insuranceJobs.getJobs().getName());
            map1.put("content",insuranceJobs.getJobs().getContent());
            map1.put("chargesY",insuranceJobs.getCharges());
            long memberCount = 1;
            long number = Long.valueOf(memberCount).intValue();
            map1.put("number",number);
            BigDecimal money = insuranceJobs.getCharges().multiply(new BigDecimal(months)).multiply(new BigDecimal(number));
            map1.put("money",money);

            totalAmount = totalAmount.add(money);
            mapList1.add(map1);
        }

        map.put("totalAmount",totalAmount);

        String cnMontrayUnit = NumberToCN.number2CNMontrayUnit(totalAmount);
        map.put("cnMontrayUnit",cnMontrayUnit);

        map.put("jobsList",mapList1);

        msgResult.setResult(true);
        msgResult.setData(map);

        return msgResult;
    }

    @ApiOperation(value="人才超市与PICC转账")
    @RequestMapping(value = "makePolicy",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="accountBank",value = "入账银行",required = true,paramType = "query"),
            @ApiImplicitParam(name="issuingBank",value = "出账银行",required = true,paramType = "query"),
            @ApiImplicitParam(name="singlNumber",value = "单号",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query")
    })
    public MessageResult<Integer> makePolicy(String applicationId,
                                           String accountBank,String issuingBank,String singlNumber,
                                           String processStatus,String opinion, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            StudentApplication studentApplication = studentApplicationService.get(applicationId);
            StudentPolicyStatus srcStudentPolicyStatus = StudentPolicyStatus.valueOf(Integer.valueOf(studentApplication.getStatus()));

            StudentPolicyStatus destStudentPolicyStatus;

            if (processStatus.equals("正常")) {
                //40
                destStudentPolicyStatus = StudentPolicyStatus.PendingOutPolicy;
            } else {
                //30
                destStudentPolicyStatus = StudentPolicyStatus.PendingPay;
            }

            studentApplication.setAccountBank(accountBank);
            studentApplication.setIssuingBank(issuingBank);
            studentApplication.setSinglNumber(singlNumber);
            studentApplication.setTransferTime(new Date());

            int affectCount = studentApplicationService.update(studentApplication);

            int affectCount1 = 0;

            if (affectCount > 0) {
                affectCount1 = approval(applicationId,  processStatus, opinion, destStudentPolicyStatus.getValue() + "", subject);
            }

            //发送微信通知
            //templateMessageService.sendTemplateMessage(applicationId, srcStudentPolicyStatus, destStudentPolicyStatus);

            msgResult.setResult(true);
            msgResult.setData(affectCount1);
        }
        catch (Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="回退转单通用")
    @RequestMapping(value = "transferOrder",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "转单步骤",required = true,paramType = "query"),
            @ApiImplicitParam(name="status",value = "转单步骤",required = true,paramType = "query")
    })
    public MessageResult<Integer> transferOrder(String applicationId,
                                                String status,
                                                String processStatus,
                                                @RequestAttribute String subject){
        //当前用户ID
        System.out.println(subject);
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            StudentApplication studentApplication = studentApplicationService.get(applicationId);
            StudentPolicyStatus srcStatus = StudentPolicyStatus.valueOf(Integer.valueOf(studentApplication.getStatus()));

            StudentPolicyStatus destStatus = StudentPolicyStatus.valueOf(Integer.valueOf(status));

            int affectCount = approval(applicationId, processStatus, "", destStatus.getValue()+ "", subject);

            //发送微信通知
            //templateMessageService.sendTemplateMessage(applicationId, srcStatus, destStatus);

            msgResult.setResult(true);
            msgResult.setData(affectCount);
        }
        catch (Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    private Integer approval(String applicationId,String processStatus,String opinion,String status,String subject){
        int affectCount = 0;

        StudentApplication studentApplication = studentApplicationService.get(applicationId);

        String curStatus = studentApplication.getStatus();

        studentApplication.setProcessStatus(processStatus);
        studentApplication.setStatus(status);
        studentApplication.setUpdateBy(subject);
        studentApplication.setUpdateTime(new Date());

        int affectCount1 = studentApplicationService.update(studentApplication);

        if(affectCount1 > 0){
            StudentApplicationRecord record = new StudentApplicationRecord();
            record.setApplicationId(studentApplication.getId());
            record.setStatus(curStatus);
            record.setOpinion(opinion);
            record.setProcessStatus(processStatus);
            record.setId(UUID.randomUUID().toString());
            record.setDelFlag(false);
            record.setCreateBy(subject);
            record.setCreateTime(new Date());

            affectCount = studentApplicationRecordService.insert(record);
        }

        return affectCount;
    }

}
